#!/usr/bin/python

import re
import time

def date2ts(ts):
	ts = time.strptime(ts, '%y%m%d%H%M%SZ')
	return (time.mktime(ts), ts)

def split_dn(dn):
	result = {}
	avp = re.sub(r'/([^/=]+)($|/)', r':slash:\1\2', dn[1:]).split('/')
	for av in avp:
		try: 
			id,value = av.split('=',1)
			if not result.has_key(id):
				result[id] = []
			result[id].append(re.sub(":slash:", "/", value))
		except:
			pass
	return result

class CInfo:
	"""
Example line:
R	040612143516Z	040715115233Z	01	unknown	/O=Grid/OU=GlobusTest/OU=simpleCA-cluster.s2s.msu.ru/OU=s2s.msu.ru/CN=Alexander V. Inyukhin
"""
	def __init__(self, line = None):
		if not line:
			return
		self.line = line
		(self.flag, self.valid_date, self.replaced_date, self.idx, u2, self.dn) = line.split('\t')
		dna = split_dn(self.dn)
		(self.valid_sec, self.valid_ts) = date2ts(self.valid_date)

		if dna.has_key("CN") and len(dna["CN"]) > 0:
			self.cn = dna["CN"][0]
		else:
			self.cn = "Unknown"
	
	def valid(self): return self.valid_sec > time.time() and self.flag == "V"
	def revoked(self): return self.flag == 'R'
	def expiring(self): 
		tm = time.time()
		return self.valid_sec > tm and self.valid_sec < tm + 86400 * 30
	def expiring_mail(self): 
		tm = time.time()
		return self.valid_sec > tm + 86400 * 6 and self.valid_sec < tm + 86400 * 7
	def match(self, str):
		return self.dn.find(str) >= 0
		
	def xml(self):
		replaced = ""
		if self.flag == "R":
			(replaced_sec, replaced_ts) = date2ts(self.replaced_date)
			replaced = "<mgp:replaced>%s</mgp:replaced>" % time.strftime('%Y/%m/%d %H:%M:%S', replaced_ts)
		return """<mgp:cert href='ca/cert/%s.pem' flag='%s'>%s
				<mgp:valid left='%d'>%s</mgp:valid>
				%s
			</mgp:cert>\n""" % (self.idx, self.flag, self.cn, (self.valid_sec - time.time())/86400, self.valid_date, replaced)

def certlist(filename):
	f = file(filename)
	for ln in f:
		yield CInfo(ln)
	f.close()

